#!/usr/bin/env python3

import os
import os.path
import re
import sys
from typing import Optional


def re_cap_fmt(src_str: str, pattern: str, dst_fmt: str) -> Optional[str]:
    """ "RE捕获 & 替换"""
    m = re.search(pattern, src_str)
    if m is None:
        return None
    for i, g in enumerate(m.groups()):
        dst_fmt = dst_fmt.replace("(%d)" % (i + 1), g)
    dst_fmt = dst_fmt.replace("(0)", m.group(0))
    return dst_fmt


def main():
    if len(sys.argv) != 4:
        print(
            r"""
    Usage:
        rename-re.py dir src_re dst_fmt
    samples:
        rename-re.py . '(\d+)_(\d+).jpg' '(2)_(1).jpg'
        rename-re.py . '(\w+).jpg' '(0)_(1).jpg'
    """
        )
        sys.exit(1)

    root_dir = sys.argv[1]  # 指明被遍历的文件夹
    pattern = sys.argv[2]  # 源RE
    repl = sys.argv[3]  # 目标格式

    for parent, dir_names, file_names in os.walk(root_dir):
        for file_name in file_names:  # 输出文件信息
            new_name = re_cap_fmt(file_name, pattern, repl)
            if new_name is not None:
                os.rename(
                    os.path.join(parent, file_name), os.path.join(parent, new_name)
                )
                # shutil.move(os.path.join(parent, file_name), os.path.join(parent, new_name))
                print("rename '%s' to '%s'" % (file_name, new_name))


if __name__ == "__main__":
    main()



from typing import Any

from rustshed import Option, Some, Null, to_option


def lookup(indexes: list, tab: list) -> list:
    """从表中查出所有索引对应的值"""
    return [tab[i] for i in indexes]


def dict_first_key(d: dict, value) -> Option[Any]:
    """字典中查找值对应的第一个键"""
    for k, v in d.items():
        if v == value:
            return Some(k)

    return Null


def low_pos(arr: list, value):
    """查找最小的插入位置"""
    for i, v in enumerate(arr):
        if value <= v:
            return i
    return len(arr)


def up_pos(arr: list, value):
    """查找最大的插入位置"""
    for i, v in enumerate(arr):
        if value < v:
            return i
    return len(arr)


@to_option
def list_index(arr: list, value) -> int:
    """List中查找值的索引，失败则Null"""
    return arr.index(value)



import re
from typing import TypeVar, Type, Any
from urllib.parse import urlparse

import redis
from pydantic import BaseModel

from jcx.text.txt_json import to_json, from_json, BMT


class DbCfg(BaseModel):
    hot_db: str
    pretty: bool = True


class RedisDb:
    """数据库变量"""

    @staticmethod
    def open(server_url: str) -> "RedisDb":
        """打开数据变量"""
        return RedisDb(server_url)

    def __init__(self, url: str):
        uri = urlparse(url)
        assert uri.scheme == "redis"
        port = uri.port or 6379

        p = re.compile(r"/(\d+)")
        m = p.match(uri.path)
        assert m
        db_num = int(m.groups()[0])
        # print(db_num)
        self._name = str(db_num)
        self._db = redis.Redis(
            host=uri.hostname, port=port, db=db_num, decode_responses=True
        )

    def name(self) -> str:
        """ "获取数据库名"""
        return self._name

    def set(self, name: str, value: Any) -> None:
        """保存变量"""
        self._db.set(name, to_json(value))

    def get(self, name: str, var_type: Type[BMT], default_value: BMT) -> BMT:
        """获取变量"""
        s = self._db.get(name)
        if not s:
            return default_value
        return from_json(s, var_type).unwrap()

    def exists(self, name: str) -> bool:
        """ "判断是否存在"""
        return self._db.exists(name) > 0

    def remove(self, name: str) -> None:
        """ "删除"""
        self._db.delete(name)


def a_test() -> None:
    cfg = DbCfg(hot_db="redis://127.0.0.1/10")

    db = RedisDb(cfg.hot_db)

    db.set("name", "jack")
    v = db.get("name", str, "")
    print(v, type(v))

    db.set("age", 18)
    v = db.get("age", int, 0)
    print(v, type(v))

    db.set("cfg", cfg)
    v = db.get("cfg", DbCfg, DbCfg())
    print(v, type(v))

    print(db.exists("cfg"))
    db.remove("cfg")
    print(db.exists("cfg"))


if __name__ == "__main__":
    a_test()



from datetime import timedelta

from arrow import Arrow


class Timer:
    """更新时间记录"""

    def __init__(self, last_updated=Arrow(1970, 1, 1)):
        self.__last_updated = last_updated  # 上一次更新时间

    def __str__(self):
        return "Timer(%s)" % str(self.__last_updated)

    def next(self, interval: timedelta) -> Arrow:
        """预计下次更新时间"""
        return self.__last_updated + interval

    def check(self, time: Arrow, interval: timedelta) -> bool:
        """检查时间是否到达更新时间"""
        n = self.next(interval)
        return time >= n

    def update(self, time: Arrow):
        """更新时间"""
        self.__last_updated = time



import math


def sqrt1(n):
    a = int(math.sqrt(n))
    b = n - a * a
    r = a + b / (2 * a)
    return r


def sqrt2(n):
    a = int(math.sqrt(n))
    b = n - a * a
    r = a + b / (2 * a) - b * b / (8 * a * a * a)
    return r


def a_num(n) -> None:
    r = math.sqrt(n)
    r1 = sqrt1(n)
    r2 = sqrt2(n)
    print("%d:\t%0.6f\t%0.10f\t%0.10f" % (n, r, r1 - r, r2 - r))


def a_test() -> None:
    a_num(85)

    a_num(101)
    a_num(10001)
    a_num(10101)
    a_num(10181)
    a_num(686756)
    a_num(909090909090909999)


if __name__ == "__main__":
    a_test()



import arrow
from arrow import Arrow

"""
参考：https://zhuanlan.zhihu.com/p/37164491
"""


def now_iso_str() -> str:
    """当前时间转字符串"""
    return datetime_iso_str(arrow.now())


def datetime_iso_str(dt: Arrow) -> str:
    """datetime转字符串"""
    return dt.strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3] + "+08:00"


def iso_to_local(iso_str: str) -> str:
    """iso格式转本地表示"""
    dt = arrow.get(iso_str)
    return datetime_local_str(dt)


def datetime_local_str(dt: Arrow, ms: bool = False) -> str:
    """datetime转字符串"""
    n = 9 if ms else 13
    return str(dt)[:-n]


def now_local_str(ms: bool = False) -> str:
    """当前时间转字符串"""
    return datetime_local_str(arrow.now(), ms)


def now_id(ms: bool = False) -> str:
    """当前时间转ID"""
    s = now_local_str(ms)
    s = s.replace(" ", "_")
    s = s.replace("T", "_")
    s = s.replace(":", "-")
    return s


def now_file(ext: str = "", ms: bool = False) -> str:
    """当前时间转字作为文件名"""
    s = now_id(ms)
    return s + ext



import hashlib
from fractions import Fraction
from typing import Any

from jcx.text.txt_json import to_json

# 实数类型
type Real = int | float

# 2维实数类型
type Real2D = tuple[Real, Real]

# 2维实数类型
type Real1_2D = Real | Real2D


def is_real(n: Any) -> bool:
    """判断变量是否为实数"""
    return isinstance(n, int) or isinstance(n, float)


def real_2d(n: Real1_2D) -> Real2D:
    """Real1_2D转为2D实数"""

    if isinstance(n, tuple):
        assert len(n) == 2
        x, y = n
    else:
        x, y = n, n
    assert is_real(x) and is_real(y)
    return x, y


def align_down(a: int, b: int) -> int:
    """ "整数向下对齐到第二个整数倍数"""
    return int(a / b) * b


def align_up(a: int, b: int) -> int:
    """ "整数向上对齐到第二个整数倍数"""
    return align_down(a + b - 1, b)


def is_multiple(a: int, b: int) -> int:
    """判定a是否是b的整数倍"""
    return a % b == 0


def align_round(n: float, align_num: int) -> int:
    """近似对齐"""
    return int(n + align_num / 2) // align_num * align_num


def a_fraction() -> None:
    f1 = Fraction(16, -10)
    print(f1)
    s = to_json(f1)
    print(s)


def md5_int(src: str, mod: int) -> int:
    """md5哈希算法返回整数"""
    md5 = hashlib.md5()
    md5.update(src.encode("utf-8"))
    n = 1
    s = 0
    for d in md5.digest():
        s = s + d * n
        n = n * 256 % mod
    return s % mod



#!/usr/bin/env python3

import argparse
import subprocess
from pathlib import Path

from loguru import logger


def main():
    parser = argparse.ArgumentParser("图像转换工具")
    parser.add_argument("file", type=Path, help="待转换文件")
    parser.add_argument("ext", type=str, help="目标文件扩展名, 如: .jpg")
    parser.add_argument("--remove", action="store_true", help="转化成功则删除源文件")
    parser.add_argument("-v", "--verbose", action="store_true", help="显示详细信息")
    opt = parser.parse_args()

    src_file: Path = opt.file  # type: ignore
    if src_file.suffix == opt.ext:
        return

    src_size = src_file.stat().st_size
    if src_size < 1000:
        logger.error(f"源文件尺寸无效: {src_size}")
        return

    dst_file: Path = src_file.with_suffix(opt.ext)  # type: ignore
    if not dst_file.exists():
        r = subprocess.call(["convert", src_file, dst_file])
        if r != 0:
            logger.error(f"转换失败, r={r}, file={dst_file}")
            return
    if not dst_file.exists():
        logger.error(f"目标路径不存在: {dst_file}")
        return

    r = subprocess.call(["identify", dst_file])
    if r != 0:
        logger.error(f"验证失败: {r}")
        return

    dst_size = dst_file.stat().st_size
    if dst_size < 1000:
        logger.error(f"目标文件尺寸无效: {dst_size}")
        return

    radio = int(dst_size / src_size * 100)
    logger.debug(f"{src_file}({radio}%)")

    if opt.remove:
        src_file.unlink()
        logger.info(f"{src_file} 删除!")


if __name__ == "__main__":
    main()



from typing import Iterator

from jcx.db.ivariant import IVariant


class Counter(Iterator[int]):
    """数据库计数器"""

    def __next__(self) -> int:
        """获取下一个数值"""
        v = self.get() + 1
        self.__var.set(v)
        return v

    def __init__(self, var: IVariant[int]):
        self.__var = var

    def name(self) -> str:
        """ "获取计数器名"""
        return self.__var.name()

    def get(self) -> int:
        """获取当前数值"""
        return self.__var.get() or 0

    def reset(self) -> None:
        """重置ID初值"""
        self.__var.set(0)



from pydantic import BaseModel


def complete(a: BaseModel, b: BaseModel) -> BaseModel:
    """用a的值补全b缺失的值"""
    for k in b.__dict__.keys():
        b.__dict__[k] = b.__dict__[k] or a.__dict__[k]
    return b



from typing import Optional, TypeVar

from jcx.rs.proto import Cloned
from rustshed import Option, Null, Some

T = TypeVar("T")

C = TypeVar("C", bound=Cloned)


def rs_option(v: Optional[T]) -> Option[T]:
    """Optional => Option"""
    if v is None:
        return Null
    return Some(v)


def rs_option_cloned(v: Optional[C]) -> Option[C]:
    """Optional => Option & cloned"""
    if v is None:
        return Null
    return Some(v.clone())


def py_optional(v: Option[T]) -> Optional[T]:
    """Option => Optional"""
    if v.is_null():
        return None
    return v.unwrap()



import os
from typing import Any
from urllib.parse import urlparse

import paho.mqtt.client as mqtt  # type: ignore
from jcx.net.mqtt.cfg import MqttCfg
from loguru import logger


class Subscriber:
    """MQTT消息订阅者"""

    def __init__(self, cfg: MqttCfg):
        """创建MQTT消息订阅者"""
        uri = urlparse(cfg.server_url)
        strs = uri.netloc.split(":", 1)
        self.url = cfg.server_url
        self.params = {
            "transport": uri.scheme,
            "hostname": strs[0],
            "port": int(strs[1]),
        }
        self.root_topic = cfg.root_topic
        self.client = mqtt.Client(protocol=mqtt.MQTTv5)
        self.client.on_message = on_message

    def dispatch_msg(self, topic: str, user_object: Any, timeout: float = 8.0) -> None:
        """分发消息"""
        self.client.user_data_set(user_object)

        topic = os.path.join(self.root_topic, topic)
        logger.info("Subscribe mqtt topic: %s/%s ..." % (self.url, topic))

        self.client.connect(self.params["hostname"], self.params["port"], 60)
        self.client.subscribe(topic, 0)

        self.client.loop_forever(timeout=timeout)


def on_message(_client, user_object: Any, msg: mqtt.MQTTMessage) -> None:
    if user_object is not None:
        user_object.on_mqtt_message(msg)


def a_test() -> None:
    class Outputer:
        def on_mqtt_message(self, mqtt_msg: mqtt.MQTTMessage) -> None:
            assert self
            print("Outputer:", mqtt_msg.payload)

    outputer = Outputer()

    cfg = MqttCfg("tcp://localhost:1883", "howell")
    subcriber = Subscriber(cfg)

    # subcriber.loop(lambda msg: print(msg))
    subcriber.dispatch_msg("1/#", outputer)


if __name__ == "__main__":
    a_test()



from copy import deepcopy
from typing import Protocol, TypeVar

C = TypeVar("C", bound="Cloned")


class Cloned(Protocol):
    """类型协议 - 复制"""

    def clone(self: C) -> C:
        """克隆记录"""
        return deepcopy(self)



#!/usr/bin/env python3

import argparse
import glob
import os
import sys
from collections import defaultdict
from os.path import splitext

from jcx.sys.fs import StrPath, dirs_in, insert_dir


def main() -> None:
    parser = argparse.ArgumentParser(description="文件系统工具箱")
    parser.add_argument("path", metavar="PATH", type=str, help="目录")
    parser.add_argument(
        "-s",
        "--same-name-count",
        type=int,
        help="重名指定次数的文件，同目录，不计扩展名",
    )
    parser.add_argument(
        "-d", "--include-dir", action="store_true", default=False, help="是否包含目录"
    )
    parser.add_argument("-v", "--verbose", action="store_true", help="显示详细信息")
    opt = parser.parse_args()

    if os.path.isdir(opt.path):
        s = os.path.join(opt.path, "**/*")
        files = sorted(glob.glob(s, recursive=True))
    else:
        print("目录不存在：", opt.path)
        sys.exit(0)

    if not opt.include_dir:
        files = [f for f in files if not os.path.isdir(f)]

    counters = defaultdict(int)
    for f in files:
        f0 = splitext(f)[0]
        counters[f0] += 1

    for f in files:
        f0 = splitext(f)[0]
        if counters[f0] == opt.same_name_count:
            print(f)


def insert_channel(folder: StrPath) -> None:
    """给抓图目录插入channel"""

    dirs = dirs_in(folder)
    for i, d in enumerate(dirs):
        print(f"#{i:04}  {d}")
        insert_dir(d, "0")


if __name__ == "__main__":
    main()
    # insert_channel('/var/ias/snapshot/shtm/n1')



from threading import Thread, Lock
from time import sleep

# 参考: https://docs.python.org/zh-cn/3/library/threading.html?highlight=threading


num = 1
lock = Lock()


def change_num():
    global num
    for i in range(100_000):
        # lock.acquire()
        num += 5
        num -= 5
        # lock.release()
        # if i % 1000 == 0:
        # print(i)


def num_test():
    pool = [Thread(target=change_num) for i in range(50)]
    for t in pool:
        t.start()
    for t in pool:
        t.join()
    print("num:", num)
    print("结论: 一般情况下不需要锁")


queue = []

N = 100_000


def queue_push():
    global queue
    for i in range(N):
        queue.append(i)


def queue_pop():
    n = 0
    global queue
    for i in range(N):
        while not queue:
            sleep(0.00001)
        v = queue.pop(0)
        if i % 1000 == 0:
            print("n:", len(queue))
        assert v == i
    print("last n:", n)


def queue_test():
    threads = [Thread(target=queue_push), Thread(target=queue_pop)]

    for t in threads:
        t.start()

    for t in threads:
        t.join()

    print("结论: 一般情况下不需要锁")


if __name__ == "__main__":
    # num_test()
    queue_test()



from pydantic import BaseModel


class Endpoint(BaseModel):
    host: str
    port: int

    def __str__(self) -> str:
        return "%s:%d" % (self.host, self.port)


class ApiCfg(BaseModel):
    name: str
    root: str
    endpoint: Endpoint

    def url(self, scheme: str) -> str:
        """获取URL"""
        return "%s://%s/%s" % (scheme, self.endpoint, self.root)


def test_cfg() -> None:
    _cfg = ApiCfg(
        name="配置", root="/howell/ias", endpoint=Endpoint(host="localhost", port=5000)
    )
    assert _cfg.name == "配置"



from pathlib import Path
from subprocess import getstatusoutput

from flask_restx import Api, fields  # type: ignore
from pydantic import BaseModel

from jcx.api.dao_item import ItemDao, add_item_resource
from jcx.time.dt import now_iso_str
from rustshed import Option, Null


class CommandInfo(BaseModel):
    """命令信息"""

    time: str = ""
    """时间"""
    output: str = ""
    """命令输出"""


class CommandDao(ItemDao):
    """数据源组数据访问对象"""

    def __init__(self, folder: Path, name: str, command: str):
        super().__init__(CommandInfo, folder, name)
        self._command = command

    def before_update(self, r: CommandInfo, _r0: CommandInfo):
        """更新之前"""
        r.time = now_iso_str()
        status, output = getstatusoutput(self._command)
        r.output = "%s, %s" % (status, output)
        return r


def command_model(api: Api, description):
    return api.model(
        "Command",
        {
            "time": fields.String(description="命令执行结束时间，PUT时忽略"),
            "output": fields.String(description=description),
        },
    )


class CommandParam(BaseModel):
    """命令参数"""

    time: str = ""
    """时间"""
    output: str = ""
    """输出?"""


def add_command_resource(
    api, ns, url, db_root: Path, cmd: str, desc: str, name: Option[str] = Null
) -> None:
    """添加命令资源"""
    name1 = name.unwrap_or(Path(url).name)
    dao = CommandDao(db_root, name1, cmd)
    add_item_resource(ns, url, dao, command_model(api, desc))



import argparse
import shutil
from pathlib import Path

from jcx.sys.fs import files_in


def main() -> None:
    parser = argparse.ArgumentParser("从参考目录选择文件名, 从候选目录选出文件")
    parser.add_argument("ref_dir", type=Path, help="参考文件目录")
    parser.add_argument("candi_dir", type=Path, help="候选文件目录")
    parser.add_argument("dst_dir", type=Path, help="候选文件目录")
    parser.add_argument(
        "-r", "--ref_ext", type=str, default="json", help="参考文件扩展名"
    )
    parser.add_argument(
        "-o", "--candi_ext", type=str, default="jpg", help="候选文件扩展名"
    )
    parser.add_argument("-v", "--verbose", action="store_true", help="显示详细信息")
    opt = parser.parse_args()

    assert Path(opt.ref_dir).is_dir(), "参考文件目录不存在: " + opt.ref_dir
    assert Path(opt.candi_dir).is_dir(), "候选文件目录不存在: " + opt.candi_dir

    mates = files_in(opt.ref_dir, opt.ref_ext)
    names = [f.stem for f in mates]

    for i, name in enumerate(names):
        f = Path(opt.candi_dir, name + "." + opt.candi_ext)
        assert Path(f).is_file(), f"文件不存在: {f}"
        shutil.copy(f, opt.dst_dir)


if __name__ == "__main__":
    main()



from typing import TypeVar, Generic, Mapping, Iterator, Optional
from pydantic import BaseModel

KT = TypeVar("KT")
VT = TypeVar("VT")


class LictItem(BaseModel, Generic[KT, VT]):
    """Lict条目"""

    key: KT  # 键
    value: VT  # 值


type LictItems = list[LictItem[KT, VT]]
"""Lict条目数组"""


class LictIter(Iterator[KT]):
    """Lict迭代器"""

    def __init__(self, items: list[LictItem[KT, VT]]):
        self._items = items
        self._idx = -1

    def __next__(self) -> KT:
        self._idx += 1
        if self._idx < len(self._items):
            return self._items[self._idx].key
        raise StopIteration


class Lict(Mapping[KT, VT]):
    """列表字典, 受某种限制(如: flask_restx), 必须用list实现map功能"""

    def __init__(self, items: list[LictItem[KT, VT]]):
        self._items = items

    def inner(self) -> list[LictItem[KT, VT]]:
        """获取内部表示"""
        return self._items

    def index(self, key: KT) -> int:
        """获取 key 所在索引"""
        for i, item in enumerate(self._items):
            if item.key == key:
                return i
        return -1

    def __getitem__(self, key: KT) -> VT:
        """获取 key 对应的值"""
        i = self.index(key)
        if i > -1:
            return self._items[i].value
        raise KeyError(f"Key {key} not found.")

    def __setitem__(self, key: KT, value: VT) -> None:
        """获取 key 对应的值"""
        i = self.index(key)
        if i > -1:
            self._items[i].value = value
        self._items.append(LictItem[KT, VT](key=key, value=value))

    def pop(self, key: KT, default_value: Optional[VT] = None) -> Optional[VT]:
        """删除 key 对应的值, 并返回"""
        i = self.index(key)
        if i > -1:
            return self._items.pop(i).value
        return default_value

    def __iter__(self) -> Iterator[KT]:
        return LictIter(self._items)

    def __len__(self) -> int:
        """获取元素数量"""
        return len(self._items)

    def to_dict(self) -> dict[KT, VT]:
        """转化成普通字典"""
        return {p.key: p.value for p in self._items}



from typing import Self

from arrow import Arrow
from parse import parse  # type: ignore
from pydantic.dataclasses import dataclass
from rustshed import Option, Some, Null


# @total_ordering
@dataclass(frozen=True, order=True)
class ClockTime:
    """时钟时间（时分秒）"""

    hour: int = 0
    """小时"""
    minute: int = 0
    """分钟"""
    second: int = 0
    """秒"""

    class Config:
        allow_mutation = False

    @classmethod
    def new(cls, hour: int, minute: int, second: int) -> Self:
        """构造"""
        return ClockTime(hour=hour, minute=minute, second=second)

    @classmethod
    def from_secs(cls, secs: int) -> Self:
        """构造"""
        minute, second = divmod(secs, 60)
        hour, minute = divmod(minute, 60)
        return ClockTime(hour=hour, minute=minute, second=second)

    @staticmethod
    def from_time(t: Arrow) -> "ClockTime":
        """datetime转ClockTime"""
        return ClockTime(hour=t.hour, minute=t.minute, second=t.second)

    @staticmethod
    def parse(s: str) -> Option["ClockTime"]:
        """从字符串解析时间"""
        arr = parse("{:d}:{:d}:{:d}", s)
        if arr:
            return Some(ClockTime(hour=arr[0], minute=arr[1], second=arr[2]))
        return Null

    def __str__(self) -> str:
        return "%02d:%02d:%02d" % (self.hour, self.minute, self.second)

    def to_time(self) -> Arrow:
        """ClockTime转datetime"""
        t = Arrow.now()
        # now.date()
        return t.replace(
            hour=self.hour, minute=self.minute, second=self.second, microsecond=0
        )


type ClockTimes = list[ClockTime]  # 时钟时间列表


def to_clock_time(time: ClockTime | str | Arrow) -> Option[ClockTime]:
    """转化成ClockTime"""
    if isinstance(time, ClockTime):
        return Some(time)
    if isinstance(time, str):
        return ClockTime.parse(time)
    elif isinstance(time, Arrow):
        return Some(ClockTime.from_time(time))

    return Null



#!/usr/bin/python3

import datetime

import paho.mqtt.client as mqtt


def on_connect(mqttc, obj, rc):
    print("OnConnetc, rc: " + str(rc))


def on_publish(mqttc, obj, mid):
    print("OnPublish, mid: " + str(mid))


def on_subscribe(mqttc, obj, mid, granted_qos):
    print("Subscribed: " + str(mid) + " " + str(granted_qos))


def on_log(mqttc, obj, level, string):
    print("Log:" + string)


def on_message(mqttc, obj, msg):
    curtime = datetime.datetime.now()
    strcurtime = curtime.strftime("%Y-%m-%d %H:%M:%S")
    print(strcurtime + ": " + msg.topic + " " + str(msg.qos) + " " + str(msg.payload))


if __name__ == "__main__":
    mqttc = mqtt.Client("test")
    mqttc.on_message = on_message
    # mqttc.on_connect = on_connect
    # mqttc.on_publish = on_publish
    # mqttc.on_subscribe = on_subscribe
    # mqttc.on_log = on_log

    # 设置账号密码（如果需要的话）
    # mqttc.username_pw_set(username, password=password)

    # 服务器地址
    strBroker = "localhost"
    # 通信端口
    port = 1883

    # 订阅主题名
    topic = "howell/ias/#"

    mqttc.connect(strBroker, port, 60)
    mqttc.subscribe(topic, 0)
    mqttc.loop_forever()



from abc import ABC
from pathlib import Path
from typing import TypeVar, Type, Optional

from cattr import unstructure, structure
from flask_restx import Resource  # type: ignore
from jcx.db.jdb.variant import JdbVariant
from jcx.db.record import PRecord

R = TypeVar("R", bound=PRecord)


class ItemDao(ABC):
    """条目数据访问对象"""

    def __init__(self, record_type: Type[R], folder: Path, name: str):
        self._var = JdbVariant(record_type, folder, name)

    def value_type(self) -> Type[R]:
        """获取条目类型"""
        return self._var.value_type()

    def variant(self):
        """获取数据变量"""
        return self._var

    def get(self) -> Optional[R]:
        """获取指定记录"""
        r: R = self._var.get()
        return r

    def update(self, r) -> Optional[R]:
        """更新记录"""
        r0: R = self._var.get()
        r = self.before_update(r, r0)
        if not r:
            return r
        self._var.set(r)
        self.after_update(r)
        return r

    def before_update(self, r: R, r0: R) -> Optional[R]:
        """更新记录之前的检查，可修改r值"""
        return r

    def after_update(self, r: R) -> None:
        """更新记录之后的检查，不可修改r值"""
        pass


def make_item(api, data: dict, item_type: Type[R]) -> tuple[bool, R]:
    """从数据构建条目"""
    try:
        item = structure(data, item_type)
    except TypeError as e:
        return False, api.abort(400, str(e))
    return True, item


def add_item_resource(ns, url, dao: ItemDao, model):
    """添加独立资源"""

    # TODO: model 通过 record_type 动态生成

    @ns.route(url)
    class Item(Resource):
        """展示所有条目列表，允许创建新条目"""

        @ns.doc("get_item")
        @ns.marshal_with(model)
        def get(self):
            """获取条目"""
            r = dao.get()
            return unstructure(r)

        @ns.expect(model)
        @ns.marshal_with(model)
        def put(self):
            """修改条目"""
            ok, r = make_item(self.api, self.api.payload, dao.value_type())
            if not ok:
                return r
            r = unstructure(dao.update(r))
            if not r:
                return self.api.abort(400, "更新条目失败")
            return r, 201



import paho.mqtt.publish as publish
import requests


def transmitMQTT(strMsg):
    strMqttBroker = "localhost"
    strMqttChannel = "test"
    # print(strMsg)
    publish.single(strMqttChannel, strMsg, hostname=strMqttBroker)


if __name__ == "__main__":
    baidu_url = "https://www.baidu.com"

    response = requests.get(baidu_url)
    body = response.content.decode()
    # print()

    transmitMQTT(body)
    print("Send msg ok.")



from abc import ABC
from typing import TypeVar, Type, Optional

from cattr import unstructure, structure
from flask_restx import Resource  # type: ignore
from jcx.db.jdb.table import Table
from jcx.db.record import PRecord
from jcx.sys.fs import StrPath

R = TypeVar("R", bound=PRecord)


class ItemListDao(ABC):
    """条目集合数据访问对象"""

    def __init__(self, record_type: Type[R], folder: StrPath):
        self._tab = Table.open(record_type, folder)

    def record_type(self) -> Type[R]:
        """获取记录类型"""
        return self._tab.record_type()

    def table(self) -> Table:
        """获取数据表"""
        return self._tab

    def records(self) -> list[R]:
        """获取所有记录"""
        return self._tab.records()

    def get(self, rid: int) -> Optional[R]:
        """获取指定记录"""
        r = self._tab.get(rid)
        return r

    def add(self, record: R) -> Optional[R]:
        """添加新记录"""
        r = self.before_add(record)
        if r is None:
            return r

        record = self._tab.add(r)
        self.after_add(record)
        return record

    def update(self, record: R) -> Optional[R]:
        """更新记录"""
        r = self.before_update(record)
        if not r:
            return r
        record = self._tab.update(r).unwrap()
        self.after_update(record)
        return record

    def remove(self, record_id: int):
        """删除记录"""
        rid = self.before_remove(record_id)
        if not rid:
            return
        self._tab.remove(rid)
        self.after_remove(rid)

    def before_add(self, record: R) -> Optional[R]:
        """添加新记录之前的检查"""
        return record

    def before_update(self, record: R) -> Optional[R]:
        """更新记录之前的检查"""
        return record

    def before_remove(self, rid: int) -> Optional[int]:
        """删除记录之前的检查"""
        assert self
        return rid

    def after_add(self, record: R):
        """添加新记录之后的检查"""
        pass

    def after_update(self, record: R):
        """更新记录之后的检查"""
        pass

    def after_remove(self, rid: int):
        """删除记录之后的检查"""
        pass


def make_record(api, data: dict, record_type: Type[R], id: int = 0) -> tuple[bool, R]:
    """从数据构建记录"""
    if "id" not in data:
        data["id"] = id
    try:
        r = structure(data, record_type)
    except TypeError as e:
        return False, api.abort(400, str(e))
    return True, r


def add_list_resource(ns, url, dao: ItemListDao, model):
    """添加集合资源"""

    # TODO: model 通过 record_type 动态生成

    @ns.route(url)
    class ItemList(Resource):
        """展示所有条目列表，允许创建新条目"""

        @ns.doc("list_items")
        @ns.marshal_list_with(model)
        def get(self):
            """列出所有条目"""
            rs = dao.records()
            return list(map(unstructure, rs))

        @ns.doc("create_item")
        @ns.expect(model)
        @ns.marshal_with(model, code=201)
        def post(self):
            """创建一个新条目"""
            ok, r = make_record(self.api, self.api.payload, dao.record_type())
            if not ok:
                return r
            r = unstructure(dao.add(r))
            if not r:
                return self.api.abort(400, "创建条目失败")
            return r, 201

    @ns.route(url + "/<int:id>")
    @ns.response(404, "指定条目未找到")
    @ns.param("id", "条目ID")
    class Item(Resource):
        """展示一个单独的条目，并允许修改和删除"""

        @ns.doc("get_item")
        @ns.marshal_with(model)
        def get(self, id):
            """获取指定条目"""
            r = unstructure(dao.get(id))
            return r or self.api.abort(404, "条目 %d 不存在" % id)

        @ns.doc("delete_item")
        @ns.response(204, "条目已经删除")
        def delete(self, id):
            """删除指定ID的条目"""
            dao.remove(id)
            return "", 204

        @ns.expect(model)
        @ns.marshal_with(model)
        def put(self, id):
            """修改指定ID的条目"""
            ok, r = make_record(self.api, self.api.payload, dao.record_type(), id)
            if not ok:
                return r
            if r.id != id:
                return self.api.abort(422, "数据中ID于URL中ID不符")
            r = unstructure(dao.update(r))
            if not r:
                return self.api.abort(400, "更新条目失败")
            return r, 201



import numpy as np
from jcx.util.algo import lookup


def random_split_n(n: int, radios: list) -> list[list]:
    """按比例随机分割整数集合[0,n-1]"""

    radios_np = np.array(radios)
    s = 0
    for i in range(len(radios_np)):
        s += radios_np[i]
        radios_np[i] = s
    radios_np = radios_np / s

    ends = (radios_np * n).astype(int)

    array = np.arange(n)
    np.random.shuffle(array)

    start = 0
    slices: list[list] = []
    for end in ends:
        slice: list = array[start:end].tolist()
        slices.append(slice)
        start = end
    return slices


def random_split_array(arr: list, radio_arr: list) -> list:
    """按比例随机分割数组"""

    slices = random_split_n(len(arr), radio_arr)
    return [lookup(s, arr) for s in slices]


def random_split(x: int | list, radio: list) -> list:
    """按比例随机分割整数集合或数组"""

    if isinstance(x, int):
        return random_split_n(x, radio)
    if isinstance(x, list):
        return random_split_array(x, radio)
    raise "Unknown type"


def random_split_test() -> None:
    slices = random_split(10, [8, 1, 1])
    for s in slices:
        print(s)

    arr = [chr(ord("a") + i) for i in range(10)]
    print(arr)

    slices = random_split(arr, [8, 1, 1])
    for s in slices:
        print(s)


def group(arr: list, group_member: int) -> list[list]:
    """分组"""

    total = len(arr)
    return [arr[i : i + group_member] for i in range(0, total, group_member)]


def group_test() -> None:
    a = [i for i in range(11)]
    print(a)
    print(group(a, 5))


if __name__ == "__main__":
    # list_index_test()
    # random_split_test()
    group_test()



from typing import Optional

from jcx.sys.fs import or_ext, StrPath
from rustshed import Result, Ok, Option, Some, Err


def load_txt(file: StrPath) -> Result[str, str]:
    """加载文本"""
    with open(file, "r") as f:
        s = f.read()
        return Ok(s)
    # return Err('load_txt fail.')


def save_txt(txt: str, file: StrPath, ext: str = ".txt") -> Result[bool, Exception]:
    """保存文本到文件"""
    file = or_ext(file, ext)
    try:
        file.parent.mkdir(parents=True, exist_ok=True)
        with open(file, "w") as f:
            f.write(txt)
    except Exception as e:
        return Err(e)
    return Ok(True)


def save_lines(
    lines: list[str], file: StrPath, ext: str = "", postfix: str = ""
) -> None:
    """多行文本保存到文件，文件自动加扩展名，自动建立目录，行尾自动加回车"""
    file = or_ext(file, ext)
    file.parent.mkdir(parents=True, exist_ok=True)
    print("save:", file)
    with open(file, "w") as f:
        for line in lines:
            f.write(line + postfix)


def replace_in_file(
    src_file: StrPath, input_: str, output: str, dst_file: Optional[StrPath] = None
) -> None:
    """文本文件替换"""
    dst_file = dst_file or src_file

    txt = load_txt(src_file).unwrap()
    new_data = txt.replace(input_, output)
    save_txt(new_data, dst_file)



import numpy as np
import torch
from rustshed import Result, Ok, Err


def detail_nda(arr: np.ndarray, type_name: str = "ndarray") -> str:
    """np.ndarray 细节"""
    assert isinstance(arr, np.ndarray)
    min1 = np.min(arr)
    max1 = np.max(arr)
    return f"{type_name}[{arr.dtype}] shape={arr.shape} value=[{min1}~{max1}]"


def detail_tensor(arr: torch.Tensor) -> str:
    """torch.Tensor 细节"""
    assert isinstance(arr, torch.Tensor)
    return detail_nda(arr.cpu().numpy(), "Tensor")


def detail(arr: np.ndarray | torch.Tensor) -> Result[str, str]:
    """显示类型细节"""
    if isinstance(arr, np.ndarray):
        return Ok(detail_nda(arr))
    elif isinstance(arr, torch.Tensor):
        return Ok(detail_tensor(arr))
    else:
        return Err(f"Unknown type: {type(arr)}")


def trace_arr(title: str, arr: np.ndarray | torch.Tensor) -> None:
    """显示类型细节"""

    s = detail(arr).unwrap()
    print(f"[{title}]: {s}")


def test_trace() -> None:
    print("")
    arr1 = np.zeros((4, 4))
    trace_arr("arr1", arr1)

    arr2 = torch.ones((4, 4))
    trace_arr("arr2", arr2)



from typing import Any, AnyStr, Optional
from typing import TypeVar, Type

import pydantic_core
from pydantic import BaseModel
from rustshed import Result, Ok, Err, result_shortcut

from jcx.sys.fs import or_ext, StrPath
from jcx.text.io import save_txt

BMT = TypeVar("BMT", bound=BaseModel)
"""BaseModel派生类型"""


def load_txt(file: StrPath, ext: str = ".txt") -> Result[str, Exception]:
    """从文件加载文本"""
    file = or_ext(file, ext)
    try:
        with open(file, "r", encoding="utf-8") as f:
            txt = f.read()
    except Exception as e:
        return Err(e)
    return Ok(txt)


def to_json(ob: Any, pretty: bool = True) -> str:
    """对象序列化为JSON"""
    indent = 4 if pretty else None
    byte_str = pydantic_core.to_json(ob, indent=indent)
    decoded_str = byte_str.decode("utf-8")
    return decoded_str


def from_json(json: AnyStr, ob_type: Type[BMT]) -> Result[BMT, Exception]:
    """从JSON文本构建对象"""
    assert isinstance(json, str | bytes), "Invalid input type @ try_from_json"
    try:
        ob = ob_type.model_validate_json(json)
    except Exception as e:
        return Err(e)
    return Ok(ob)


def save_json(obj: Any, file: StrPath, pretty: bool = True) -> Result[bool, Exception]:
    """对象序保存为JSON文件"""
    file = or_ext(file, ".json")
    s = to_json(obj, pretty)
    return save_txt(s, file)


@result_shortcut
def load_json(file: StrPath, obj_type: Type[BMT]) -> Result[BMT, Exception]:
    """从Json文件加载对象"""
    file = or_ext(file, ".json")
    s = load_txt(file).Q
    # print('load_json:', file)
    return from_json(s, obj_type)


@result_shortcut
def load_json_or(
    file: Optional[StrPath], obj_type: Type[BMT], default_value: BMT
) -> Result[BMT, Exception]:
    """从Json文件加载对象，文件路径未提供则返回默认值"""
    return load_json(file, obj_type) if file else Ok(default_value)



import argparse
from pathlib import Path

from jcx.sys.fs import move_file, replace_home
from jcx.text.io import save_lines
from jcx.time.dt import now_file


def main():
    parser = argparse.ArgumentParser("History文件清理工具")
    parser.add_argument(
        "-f", "--history-file", type=Path, default="~/.zsh_history", help="History文件"
    )
    parser.add_argument(
        "-b",
        "--backup-dir",
        type=Path,
        default="~/.config/history_arch",
        help="History文件备份目录",
    )
    parser.add_argument("-v", "--verbose", action="store_true", help="显示详细信息")
    opt = parser.parse_args()

    history_file = replace_home(opt.history_file)
    backup_dir = replace_home(opt.backup_dir)

    cmd_counts = {}
    lines = []
    num_lines = 0
    for i, line in enumerate(open(history_file)):
        num_lines += 1
        r = line.split(";", 1)
        if len(r) != 2:
            print("WARN @ %d: " % i, line)
            continue
        _, cmd = r
        if cmd not in cmd_counts:
            lines.append(line)
            cmd_counts[cmd] = 0
        cmd_counts[cmd] += 1

    for k, v in cmd_counts.items():
        if v > 1:
            print("n=", v, k)

    bak_file = backup_dir / now_file()
    print("备份历史文件到：", bak_file)
    move_file(history_file, bak_file).unwrap()

    print("历史文件，行数变化：%d -> %d" % (num_lines, len(lines)))
    save_lines(lines, history_file)


if __name__ == "__main__":
    main()



import argparse
import shutil
from pathlib import Path

from jcx.sys.fs import files_in, mtime_to_name, ctime_to_name


def main():
    parser = argparse.ArgumentParser("文件改名/复制工具")
    parser.add_argument("src_dir", type=Path, help="文件来源目录")
    parser.add_argument("dst_dir", type=Path, help="文件目标目录")
    parser.add_argument("-e", "--ext", type=str, default="jpg", help="文件扩展名")
    parser.add_argument(
        "-m",
        "--method",
        type=str,
        required=True,
        help="改名方法：ctime-改为创建时间, mtime-改为修改时间, now-改为当前时间",
    )
    parser.add_argument("-v", "--verbose", action="store_true", help="显示详细信息")
    opt = parser.parse_args()

    methods = {"ctime": ctime_to_name, "mtime": mtime_to_name}
    method = methods[opt.method]

    files = files_in(opt.src_dir, opt.ext)
    for src in files:
        name = method(src)
        dst = Path(opt.dst_dir) / name
        print(src.name, " -> ", dst.name)
        dst.parent.mkdir(parents=True, exist_ok=True)
        shutil.copy(src, dst)


if __name__ == "__main__":
    main()



import traceback
from typing import Callable, TypeVar, Optional, Any

from loguru import logger
from rustshed import Err

T = TypeVar("T")


def mand(value: Optional[T]) -> T:
    """强制可选类型非空"""
    assert value is not None
    return value


def show_err(e: Any) -> None:
    """显示错误/异常"""
    if isinstance(e, Err):
        msg = f"{e}"
    # elif isinstance(e, UnwrapError):
    #    msg = 'UnwrapError(%s):' % str(e.result.value)
    elif isinstance(e, AssertionError):
        msg = f"AssertionError({e.args})"
    else:
        msg = f"UnknownError({repr(e)})"
    logger.error(msg)


def catch_show_err(fun: Callable, verbose: bool = False) -> None:
    """捕获并显示异常"""

    # 捕获SystemExit/KeyboardInterrupt/GeneratorExit外异常
    # 想捕获这三个异常，需BaseException
    try:
        fun()
    except Exception as e:
        show_err(e)
        # print('traceback.print_exc():', traceback.print_exc())
        if verbose:
            print(traceback.format_exc())


def show_err_demo() -> None:
    e = Err("a error")
    print(type(e))
    show_err(e)


if __name__ == "__main__":
    show_err_demo()



from pathlib import Path
from typing import TypeVar, Type

from jcx.db.jdb.util import load_dict
from jcx.db.record import Record, RecordFilter
from jcx.rs.rs import rs_option_cloned
from jcx.sys.fs import rm_files_in, StrPath
from jcx.text.txt_json import save_json
from rustshed import Option, Null, Some

R = TypeVar("R", bound=Record)
"""记录类型"""


class Table:
    """数据库表"""

    @staticmethod
    def open(record_type: Type[R], folder: StrPath) -> "Table":
        """打开数据库表"""
        tab = Table(record_type)
        tab.load(folder)
        return tab

    def __init__(self, record_type: Type[R]):
        self._type = record_type
        self._folder: Option[Path] = Null
        self._records: dict[int, R] = {}

    def load(self, folder: StrPath) -> int:
        """加载数据库表"""
        folder1 = Path(folder)
        folder1.mkdir(exist_ok=True)

        self._folder = Some(folder1)
        self._records = load_dict(self._type, folder)
        return len(self._records)

    def record_type(self) -> Type[R]:
        """获取记录类型"""
        return self._type

    def __len__(self) -> int:
        return len(self._records)

    def ids(self) -> list[int]:
        """获取所有ID"""
        return list(self._records.keys())

    def records(self) -> list[R]:
        """获取所有记录，防止被外部导致磁盘数据不一致"""
        return sorted(self._records.values(), key=lambda r: r.id)

    def get(self, rid: int) -> Option[R]:
        """获取指定记录"""
        r = self._records.get(rid)
        return rs_option_cloned(r)

    def query(self, fun: RecordFilter) -> list[R]:
        """查询满足条件的记录集"""
        return [r for r in self._records.values() if fun(r)]

    def query_ids(self, fun: RecordFilter) -> list[int]:
        """查询满足条件的记录ID集"""
        return [r.id for r in self._records.values() if fun(r)]

    def add(self, record: R) -> Option[R]:
        """添加新记录"""
        if record.id in self._records:
            return Null

        record = record.clone()  # 避免影响传入对象
        if record.id < 1:
            record.id = self.next_id()

        return self.update(record)

    def update(self, record: R) -> Option[R]:
        """更新记录"""
        if record.id < 1:
            return Null
        record = record.clone()  # 避免影响传入对象
        self._records[record.id] = record
        self.__save(record)
        return Some(record.clone())

    def remove(self, rid: int) -> None:
        """更新记录"""
        if rid not in self._records:
            return
        f = self.__record_path(rid)
        f.unlink()
        self._records.pop(rid)

    def clear(self) -> None:
        """删除所有记录"""
        self._records.clear()
        rm_files_in(self._folder.unwrap(), ".json")

    def next_id(self) -> int:
        """获取下一个ID"""
        return max(self.ids(), default=0) + 1

    def __record_path(self, rid: int) -> Path:
        """获取记录对应路径"""
        return self._folder.unwrap() / f"{rid}.json"

    def __save(self, record: R) -> None:
        """保存路径"""
        f = self.__record_path(record.id)
        save_json(record, f)



from pydantic import BaseModel


class Linear(BaseModel):
    """线性函数"""

    k: float
    b: float

    @classmethod
    def from_xyxy(cls, x1: float, y1: float, x2: float, y2: float) -> "Linear":
        """构造, 用: x1, y1, x2, y2"""
        k = (y2 - y1) / (x2 - x1)
        b = y1 - k * x1
        return Linear(k=k, b=b)

    def __call__(self, x: float) -> float:
        y = self.k * x + self.b
        return y



#!/usr/bin/env python3

import argparse
import hashlib
from pathlib import Path
from typing import Dict

from pydantic import BaseModel


class FileInfo(BaseModel):
    path: Path
    md5: str


def file_map(folder: Path) -> Dict[int, FileInfo]:
    m: Dict[int, FileInfo] = {}
    for f in folder.rglob("*.*"):
        if f.is_file():
            info = FileInfo(path=f, md5=calc_md5(f))
            size = f.stat().st_size
            if size not in m:
                m[size] = []
            m[size] = info
    return m


def calc_md5(file: Path) -> str:
    m = hashlib.md5()  # 创建md5对象
    print(f"{file}")
    with open(file, "rb") as f:
        data = f.read(1 << 20)
        m.update(data)  # 更新md5对象

    return m.hexdigest()  # 返回md5对象


def main() -> None:
    parser = argparse.ArgumentParser("对照源目录, 删除目标目录的重复文件")
    parser.add_argument("src_dir", type=Path, help="文件来源目录")
    parser.add_argument("dst_dir", type=Path, help="文件目标目录")
    parser.add_argument("-v", "--verbose", action="store_true", help="显示详细信息")
    opt = parser.parse_args()

    src_map = file_map(opt.src_dir)
    dst_map = file_map(opt.dst_dir)

    for size, dst_info in dst_map.items():
        if size in src_map:
            src_info = src_map.get(size)
            assert src_info
            if src_info.md5 == dst_info.md5:
                print(f'REMOVE: "{dst_info.path}"\n    as: "{src_info.path}"')
                try:
                    dst_info.path.unlink()
                except OSError as e:
                    print(f"ERROR: {dst_info.path} {e}")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import re
import shutil
import sys
from pathlib import Path
from typing import Optional


def re_cap_fmt(src_str: str, pattern: str, dst_fmt: str) -> Optional[str]:
    """ "RE捕获 & 替换"""
    m = re.search(pattern, src_str)
    if m is None:
        return None
    for i, g in enumerate(m.groups()):
        dst_fmt = dst_fmt.replace("(%d)" % (i + 1), g)
    dst_fmt = dst_fmt.replace("(0)", m.group(0))
    return dst_fmt


def main():
    if len(sys.argv) != 4:
        print(
            r"""
    Usage:
        mv-re.py src_path src_re dst_fmt
    samples:
        mv-re.py ./2016-05-06/57 './([^/]+)/(\d+)' '../(2)/(1)'
        find -type d -exec mv-re.py {} "\./([^/]+)/(\d+)" "../(2)/(1)" \;
        find -name '*.mp4' -exec mv-re.py {} "[^_]+_(.+)" "(1)" \;
    """
        )

        sys.exit(1)

    src_path = sys.argv[1]  # 指明被遍历的文件夹
    src_re = sys.argv[2]  # 源RE
    dst_fmt = sys.argv[3]  # 目标格式

    dst_path = re_cap_fmt(src_path, src_re, dst_fmt)

    if dst_path is not None:
        Path(dst_path).parent.mkdir(exist_ok=True, parents=True)
        print("move '%s' to '%s'" % (src_path, dst_path))
        shutil.move(src_path, dst_path)


if __name__ == "__main__":
    main()



from typing import Type, Protocol, TypeVar, Generic

from rustshed import Option

T = TypeVar("T")


class IVariant(Protocol[T]):
    """数据库变量接口"""

    def name(self) -> str:
        """获取变量名"""
        pass

    def value_type(self) -> Type[T]:
        """获取变量类型"""
        pass

    def exists(self) -> bool:
        """判断是否存在"""
        pass

    def get(self) -> Option[T]:
        """获取变量"""
        pass

    def set(self, value: T) -> None:
        """设置变量"""
        pass

    def remove(self) -> None:
        """删除变量"""
        pass



import argparse
import subprocess
from pathlib import Path

from jcx.sys.fs import du
from loguru import logger

cmd_map = {".zip": ["unzip" "-O" "GBK"]}


def main():
    parser = argparse.ArgumentParser("解压缩工具")
    parser.add_argument("file", type=Path, help="待解压文件")
    parser.add_argument("--remove", action="store_true", help="转化成功则删除源文件")
    parser.add_argument("-v", "--verbose", action="store_true", help="显示详细信息")
    opt = parser.parse_args()

    src_file = Path(opt.file)
    cmd = cmd_map.get(src_file.suffix)
    if cmd is None:
        return logger.error(f"未知文件格式: {src_file}")

    src_size = src_file.stat().st_size
    if src_size < 1000:
        return logger.error(f"源文件尺寸无效: {src_file}({src_size})")

    dst_file = src_file.with_suffix("")
    if not dst_file.exists():
        cmd.append(str(src_file))
        r = subprocess.call(cmd)
        if r != 0:
            return logger.error(f"解压失败, r={r}, file={dst_file}")

    if not dst_file.exists():
        return logger.error(f"目标路径不存在: {dst_file}")

    dst_size = du(dst_file)

    radio = int(dst_size / src_size * 100)
    logger.debug(f"{src_file}({radio}%)")

    if radio < 100:
        return logger.error(f"解压文件尺寸小于源文件: {src_file}")

    if opt.remove:
        src_file.unlink()
        logger.info("源文件删除!")


if __name__ == "__main__":
    main()



from typing import TypeVar, Type

from jcx.db.rdb.db import RedisDb, DbCfg

T = TypeVar("T")


class RedisVariant:
    """数据库变量"""

    @staticmethod
    def open(value_type, db: RedisDb, name: str, default=None):
        """打开数据变量"""
        return RedisVariant(value_type, db, name, default)

    def __init__(self, value_type: Type[T], db: RedisDb, name: str, default=None):
        self._type = value_type
        self._db = db
        self._name = str(name)
        self._default = default

    def name(self):
        """ "获取变量名"""
        return self._name

    def value_type(self):
        """ "获取变量类型"""
        return self._type

    def exists(self):
        """ "判断是否存在"""
        return self._db.exists(self._name)

    def get(self):
        """获取变量"""
        return self._db.get(self._name, self._type, self._default)

    def set(self, value):
        """设置变量"""
        self._db.set(self._name, value)

    def remove(self):
        """删除变量"""
        self._db.remove(self._name)


def a_test():
    db_cfg = DbCfg("redis://127.0.0.1/10")

    db = RedisDb(db_cfg.hot_db)

    name = RedisVariant(str, db, "name")
    name.set("jack")
    v = name.get()
    print(v, type(v))

    age = RedisVariant(int, db, "age")
    age.set(18)
    v = age.get()
    print(v, type(v))

    cfg = RedisVariant(DbCfg, db, "cfg")
    cfg.set(db_cfg)
    v = cfg.get()
    print(v, type(v))

    print(cfg.exists())
    cfg.remove()
    print(cfg.exists())

    cfg = RedisVariant(DbCfg, db, "cfg1", 0)
    v = cfg.get()
    print(v, type(v))


if __name__ == "__main__":
    a_test()



from jcx.db.counter import Counter
from jcx.db.rdb.db import RedisDb, DbCfg
from jcx.db.rdb.variant import RedisVariant


class RedisCounter(Counter):
    """Redis计数器"""

    def __init__(self, db: RedisDb, name: str):
        super().__init__(RedisVariant(int, db, name))


def test_counter() -> None:
    db_cfg = DbCfg("redis://127.0.0.1/10")

    db = RedisDb(db_cfg.hot_db)

    c1 = RedisCounter(db, "test_c1")
    c1.reset()
    assert c1.get() == 0

    for i in range(1, 10):
        n = next(c1)
        assert i == n


if __name__ == "__main__":
    test_counter()



from pathlib import Path
from typing import TypeVar, Type, Optional

from jcx.db.record import Record, RecordFilter
from jcx.sys.fs import StrPath
from jcx.text.txt_json import load_json

R = TypeVar("R", bound=Record)


def load_list(
    record_type: Type[R], folder: StrPath, filter_: Optional[RecordFilter] = None
) -> list[R]:
    """加载记录到列表"""
    records: list[R] = []

    folder = Path(folder)
    if not folder.is_dir():
        return records

    for f in folder.glob("*.json"):
        r = load_json(f, record_type).unwrap()
        if filter_ and not filter_(r):
            continue
        records.append(r)
    return records


def load_dict(
    record_type: Type[R], folder: StrPath, filter_: Optional[RecordFilter] = None
) -> dict[int, R]:
    """加载记录到字典"""
    rs = load_list(record_type, folder, filter_)
    return {r.id: r for r in rs}



class ProgressMeter(object):
    def __init__(self, num_batches, meters, prefix=""):
        self.batch_fmtstr = self._get_batch_fmtstr(num_batches)
        self.meters = meters
        self.prefix = prefix

    def display(self, batch):
        entries = [self.prefix + self.batch_fmtstr.format(batch)]
        entries += [str(meter) for meter in self.meters]
        print("\t".join(entries))

    def _get_batch_fmtstr(self, num_batches):
        assert self
        num_digits = len(str(num_batches // 1))
        fmt = "{:" + str(num_digits) + "d}"
        return "[" + fmt + "/" + fmt.format(num_batches) + "]"



#!/opt/ias/env/bin/python

import argparse
from pathlib import Path

from jcx.sys.fs import files_in


def has_mate(file: Path, names: list) -> bool:
    """检查文件是否有伙伴文件"""
    for name in names:
        # print('math:', name, file.stem)
        if name.startswith(file.stem):
            return True
    return False


def main() -> None:
    parser = argparse.ArgumentParser("文件查找工具")
    parser.add_argument("src_dir", type=Path, help="文件来源目录")
    parser.add_argument("-e", "--ext", type=str, default="jpg", help="文件扩展名")
    parser.add_argument(
        "-m", "--mate_ext", type=str, default="lbl", help="文件伙伴文件扩展名"
    )
    parser.add_argument(
        "-i", "--invert_match", action="store_true", help="选中不匹配的文件"
    )
    parser.add_argument("-v", "--verbose", action="store_true", help="显示详细信息")
    opt = parser.parse_args()

    mates = files_in(opt.src_dir, opt.mate_ext)
    names = [f.stem for f in mates]

    candidates = files_in(opt.src_dir, opt.ext)

    for f in candidates:
        if has_mate(f, names) != opt.invert_match:
            print(f)


if __name__ == "__main__":
    main()



from pathlib import Path

from pydantic import BaseModel

from jcx.db.counter import Counter
from jcx.db.jdb.variant import JdbVariant


class Item(BaseModel):
    """数据库记录"""

    id: int


class JdbCounter(Counter):
    """Jdb计数器"""

    def __init__(self, db: Path, name: str):
        super().__init__(JdbVariant(int, db, name))



from typing import Callable, Self

from pydantic import BaseModel


class Record(BaseModel):
    """数据库记录"""

    id: int
    """记录ID"""

    def clone(self: Self) -> Self:
        """克隆记录"""
        return self.model_copy(deep=True)


type RecordFilter = Callable[[Record], bool]
"""记录过滤器"""



from pydantic import BaseModel
from rustshed import Result, Ok, Err

from jcx.time.clock_time import ClockTime


class ClockPeriod(BaseModel):
    """时钟时间段"""

    begin: ClockTime = ClockTime()
    """起始时间"""
    end: ClockTime = ClockTime()
    """截至时间"""

    class Config:
        frozen = True

    def __str__(self) -> str:
        return "[%s,%s)" % (self.begin, self.end)

    def __contains__(self, clock_time: ClockTime) -> bool:
        return self.begin <= clock_time < self.end


type ClockPeriods = list[ClockPeriod]
"""时钟时间段集合"""


class CalendarTrigger(BaseModel):
    """日程表触发器"""

    periods: ClockPeriods
    """触发时段集合"""

    class Config:
        frozen = True

    def start_time(self) -> ClockTime:
        """日程的开始时间"""
        return self.periods[0].begin if self.periods else ClockTime()

    def check(self, clock_time: ClockTime) -> bool:
        """判定时间是否满足日历触发条件"""
        # 时段检查
        ok = False
        if self.periods:
            for p in self.periods:
                if clock_time in p:
                    ok = True
                    break
        else:
            ok = True
        # 检查星期 TODO:
        return ok

    def valid(self) -> Result[bool, str]:
        """判断是否有效"""
        if len(self.periods) > 0:
            return Ok(True)
        return Err("日程表触发器时段不存在")


type CalendarTriggers = list[CalendarTrigger]  # 时钟时间段集合



from pydantic import BaseModel


class Counter(BaseModel):
    """计数器"""

    count: int = 0
    """计数值"""

    def __str__(self) -> str:
        return "Counter(%d)" % self.count

    def get(self) -> int:
        """获取计数值"""
        return self.count

    def get_inc(self) -> int:
        """获取计数值并更新计数器"""
        t = self.get()
        self.count += 1
        return t


class CountTimer(BaseModel):
    """计数闹钟"""

    interval: int
    """间隔"""
    count: int = 0
    """计数值"""

    def __str__(self) -> str:
        return "CountTimer(%d,%d)" % (self.interval, self.count)

    def inc(self) -> None:
        """更新计数器"""
        self.count += 1

    def check(self) -> bool:
        """计数器是否到规定的间隔"""
        return self.count >= self.interval

    def inc_check(self) -> bool:
        """计数器累计，并检查是否到规定的间隔"""
        self.inc()
        return self.check()

    def auto_check(self) -> bool:
        """计数器累计，并检查是否到规定的间隔，到规定时间则自动复位"""
        ok = self.inc_check()
        if ok:
            self.reset()
        return ok

    def reset(self) -> None:
        """计数器复位"""
        self.count = 0



class AverageMeter(object):
    """Computes and stores the average and current value"""

    def __init__(self, name, fmt=":f"):
        self.name = name
        self.fmt = fmt
        self.val = 0
        self.avg = 0
        self.sum = 0
        self.count = 0
        self.reset()

    def reset(self):
        self.val = 0
        self.avg = 0
        self.sum = 0
        self.count = 0

    def update(self, val, n=1):
        self.val = val
        self.sum += val * n
        self.count += n
        self.avg = self.sum / self.count

    def __str__(self):
        fmtstr = "{name} {val" + self.fmt + "} ({avg" + self.fmt + "})"
        return fmtstr.format(**self.__dict__)



import threading

import redis  # type: ignore

conn = redis.Redis(host="localhost", port=6379, db=1)
conn.set("num", 1)


def change_num(conn1):
    print("thread start")
    for i in range(10000):
        conn1.incr("num", 5)
        conn1.decr("num", 5)


if __name__ == "__main__":
    conn_pool = [
        redis.StrictRedis(host="localhost", port=6379, db=1) for i in range(16)
    ]
    t_pool = []
    for conn in conn_pool:
        t = threading.Thread(target=change_num, args=(conn,))
        t_pool.append(t)
    for t in t_pool:
        t.start()
    for t in t_pool:
        t.join()
    print(conn.get("num"))



from random import shuffle
from typing import Optional, Any


def random_choices(arr: list[Any], n: int, excluded: Optional[Any] = None) -> list[Any]:
    """数组中选出指定数量的其他元素"""
    arr1 = list(arr)
    if excluded is not None:
        arr1 = [a for a in arr1 if a != excluded]
    shuffle(arr1)
    n = min(n, len(arr1))
    return arr1[:n]



#!/usr/bin/env python3

import argparse
import glob
import os
import sys
from shutil import copyfile


def folder_files(folder, ext):
    if os.path.isdir(folder):
        s = os.path.join(folder, "**/*." + ext)
        files = glob.glob(s, recursive=True)
        files.sort()
    else:
        print("数据源不存在:", folder)
        sys.exit(0)
    return files


def copy_file(src, dst):
    try:
        copyfile(src, dst)
        print(" ", dst)
    except IOError as e:
        print("无法复制文件：%s 错误信息：%s" % (src, e))


def main() -> None:
    parser = argparse.ArgumentParser("文件复制工具")
    parser.add_argument("src_dir", type=str, help="文件来源目录")
    parser.add_argument("dst_dir", type=str, help="文件目标目录")
    parser.add_argument("-e", "--ext", type=str, default="jpg", help="文件扩展名")
    parser.add_argument(
        "-r", "--retain_num", type=int, default=5, help="保留的路径节数"
    )
    parser.add_argument(
        "-m", "--make_date_dir", action="store_true", help="创建日期目录"
    )
    parser.add_argument("-v", "--verbose", action="store_true", help="显示详细信息")
    opt = parser.parse_args()

    files = folder_files(opt.src_dir, opt.ext)

    for src in files:
        parts = src.split("/")[-opt.retain_num :]
        name = "_".join(parts)
        dst = os.path.join(opt.dst_dir, name)
        copy_file(src, dst)


if __name__ == "__main__":
    main()



from pathlib import Path
from typing import Type, Self

from jcx.db.ivariant import IVariant
from jcx.sys.fs import StrPath
from jcx.text.txt_json import load_json, save_json, BMT


class JdbVariant(IVariant):
    """数据库变量"""

    @classmethod
    def open(cls, value_type: Type[BMT], folder, name: str) -> Self:
        """打开数据变量"""
        return JdbVariant(value_type, folder, name)

    def __init__(self, value_type: Type[BMT], folder: StrPath, name: str):
        self._type = value_type
        self._path = Path(folder, name + ".json")

    def name(self) -> str:
        """ "获取变量名"""
        return self._path.stem

    def value_type(self) -> Type[BMT]:
        """ "获取变量类型"""
        return self._type

    def exists(self) -> bool:
        """ "判断是否存在"""
        return self._path.exists()

    def get(self) -> BMT:
        """获取变量"""
        return load_json(self._path, self._type).unwrap()

    def set(self, value: BMT) -> None:
        """设置变量"""
        save_json(value, self._path)

    def remove(self) -> None:
        """删除变量"""
        Path.unlink(self._path)



import os
import re
import shutil
import sys
from enum import Enum
from pathlib import Path
from typing import Generator, Any, Callable, Optional

import arrow
import sh  # type: ignore
from arrow import Arrow
from loguru import logger
from parse import parse  # type: ignore
from rustshed import Err, Result, Ok, Option, Some, Null

type Paths = list[Path]
"""路径数组"""

type StrPath = str | Path
"""路径表示：str或者Path"""

type StrPaths = list[StrPath]
"""可表示路径类型的数组"""


class Order(Enum):
    ASC = 1
    DESC = 2


def first_file_in(folders: StrPaths, file_name: str) -> Option[Path]:
    """从一组目录中查找指定文件，只获取第一个"""
    for p in folders:
        f = Path(p) / file_name
        if f.is_file():
            return Some(f)
    return Null


def files_in(folder: StrPath, ext: str, reverse: bool = False) -> list[Path]:
    """获取文件夹内指定扩展名的文件，有序"""
    p = Path(folder)
    return sorted(p.glob("*" + ext), reverse=reverse)


def get_project_dir(bin_file: StrPath) -> Path:
    """获取项目目录 - 通过可执行文件路径"""
    return Path(bin_file).parent.parent.parent.resolve()


def file_names_in(folder: StrPath, ext: str, reverse: bool = False) -> list[str]:
    """获取文件夹内指定扩展名的文件名称，有序"""
    return [f.name for f in files_in(folder, ext, reverse)]


def find(src: StrPath, ext: str, order: Order = Order.ASC) -> list[Path]:
    """查找文件或文件夹内指定扩展名文件"""
    src = Path(src)
    if src.is_dir():
        files = sorted(src.rglob("*" + ext), reverse=(order == Order.DESC))
    elif src.is_file():
        files = [src]
    else:
        files = []
    return files


def find_first(
    folder: StrPath, pattern: str, recursive: bool = True
) -> Result[Path, str]:
    """在文件夹内查找满足条件的第一个文件"""
    folder = Path(folder)
    if not folder.is_dir():
        return Err("指定路径不是目录: " + str(folder))

    it = folder.rglob(pattern) if recursive else folder.glob(pattern)
    try:
        f = next(it)
    except StopIteration:
        return Err("指定文件找不到: " + pattern)
    return Ok(f)


def find_in_parts(folder: StrPath, sub_path: str) -> Option[Path]:
    """在路径的各个部分里查找指定路径"""
    folder = Path(folder).absolute()
    while True:
        path = folder / sub_path
        if path.exists():
            return Some(path)
        folder = folder.parent
        if folder == folder.parent:
            break
    return Null


def file_exist_in(folder: StrPath, pattern: str, recursive: bool = False) -> bool:
    """判定目录中是否存在指定扩展名的文件"""
    return find_first(folder, pattern, recursive).is_ok()


def dirs_in(folder: StrPath, order: Order = Order.ASC) -> list[Path]:
    """获取文件夹"""
    folder = Path(folder)
    dirs: Paths = []
    if not folder.is_dir():
        return dirs
    for f in folder.iterdir():
        if f.is_dir():
            dirs.append(f)
    if order:
        return sorted(dirs, reverse=(order == Order.DESC))
    return dirs


def find_descendants(folder: StrPath, pattern: str, generation: int) -> list[Path]:
    """查找匹配模式的指定代子孙文件/目录"""
    assert generation > 0
    folder = Path(folder)

    if generation == 1:
        return sorted(folder.glob(pattern))

    children = dirs_in(folder)
    descendants = []
    for child in children:
        d = find_descendants(child, pattern, generation - 1)
        descendants.extend(d)
    return descendants


def rm_files_in(folder: StrPath, ext: str) -> None:
    """删除文件夹内指定扩展名文件"""
    for f in Path(folder).glob("*" + ext):
        Path(f).unlink()


def remake_dir(parent: StrPath, name: str) -> Path:
    """删除并重建目录"""
    path = Path(parent, name)
    if path.exists():
        shutil.rmtree(path)
    path.mkdir(parents=True)
    return path


def make_parents(p: StrPath) -> Path:
    """创建上级目录"""
    p = Path(p).parent
    p.mkdir(parents=True, exist_ok=True)
    return p


def or_ext(file: StrPath, ext: str) -> Path:
    """如果不存在，则补充扩展名"""
    file = Path(file)
    if not file.suffix and ext:
        file = file.with_suffix(ext)
    return file


def last_parts(file: StrPath, n: int) -> Path:
    """获取路径最后的n部分"""
    p = Path(file)
    return Path(*p.parts[-n:])


def with_parent(file: StrPath, parent: str) -> Path:
    """替换上级目录名"""
    file = Path(file)
    return file.parent.parent / parent / file.name


def find_pattern(folder: StrPath, ext: str, pattern: str) -> Generator[Path, Any, None]:
    """查找匹配指定模式的文件"""
    for f in Path(folder).rglob("*" + ext):
        if re.search(pattern, str(f)):
            yield f


def time_to_file(time: Arrow, ext: str, date_dir: bool = True) -> str:
    """时间转为文件名"""
    s = "/" if date_dir else "_"
    fmt = "%Y-%m-%d" + s + "%H-%M-%S.%f"
    name = time.strftime(fmt)[:-3] + ext
    return name


def device_time_file(
    folder: Path, dev_id: int | str, time: Arrow, ext: str, date_dir: bool = True
) -> Path:
    """根据设备ID,所在目录创建时间为名称的文件"""
    file = time_to_file(time, ext, date_dir)
    p = Path(folder, str(dev_id), file)
    make_parents(p)
    return p


def not_file_to_time(path: StrPath) -> Arrow:
    """文件名转时间 2023-04-10_10-09-39.830"""
    p = Path(path)
    s = "%sT%s+%s" % (p.parent.name, p.stem.replace("-", ":"), "08:00")
    return arrow.get(s)


def link_files(
    src_files: list[Path],
    dst_dir: Path,
    check_fun: Optional[Callable[[Path], bool]] = None,
) -> None:
    """链接文件列表到目标目录"""
    for f in src_files:
        dst = dst_dir / f.name
        dst.parent.mkdir(exist_ok=True, parents=True)
        dst.symlink_to(f.absolute())
        if check_fun and not check_fun(dst):
            logger.error(f"check fail: {dst}")


def real_path(path: StrPath) -> Path:
    """ "获取真实文件"""
    p = Path(path)
    return Path(os.readlink(p)) if p.is_symlink() else p


def real_exe_path() -> Path:
    """ "获取真实可执行文件路径"""
    return real_path(sys.argv[0])


def copy_file(src: Path, dst: Path) -> Result[Path, IOError]:
    """复制文件"""
    dst.parent.mkdir(parents=True, exist_ok=True)
    try:
        shutil.copyfile(src, dst)
    except IOError as e:
        return Err(e)
    return Ok(dst)


def move_file(src: Path, dst: Path) -> Result[Path, IOError]:
    """移动文件"""
    dst.parent.mkdir(parents=True, exist_ok=True)
    try:
        shutil.move(src, dst)
    except IOError as e:
        return Err(e)
    return Ok(dst)


def insert_dir(folder: StrPath, dir_name: str) -> None:
    """在目录中插入一级目录, 原目录内的文件移动如新目录"""
    folder = Path(folder)
    assert folder.is_dir()
    tmp = folder.parent / (folder.name + "_tmp")
    dst = folder / dir_name
    shutil.move(folder, tmp)

    assert not folder.exists()
    folder.mkdir()
    assert folder.exists()
    shutil.move(tmp, dst)


def file_ctime(file: StrPath) -> Arrow:
    """获取文件的建立时间"""
    t = Path(file).stat().st_ctime  # 创建时间
    return Arrow.fromtimestamp(t)


def file_mtime(file: StrPath) -> Arrow:
    """获取文件的修改时间"""
    t = Path(file).stat().st_mtime  # 创建时间
    return Arrow.fromtimestamp(t)


def ctime_to_name(src: Path) -> str:
    """以文件创建时间生成文件名"""
    return time_to_file(file_ctime(src), src.suffix)


def mtime_to_name(src: Path) -> str:
    """以文件修改时间生成文件名"""
    return time_to_file(file_mtime(src), src.suffix)


'''
_file_now: Option[Arrow] = Null  # TODO: 用Iter对象重写

def now_to_name(src: Path) -> str:
    """以当前文件时间生成文件名"""
    global _file_now
    match _file_now:
        case Null:
            _file_now = file_mtime(src)
        case Some(v):
            v = _file_now.shift(seconds=1)
    return time_to_file(_file_now.unwrap(), src.suffix)
'''


def stem_append(p: Path, s: str) -> Path:
    """文件名主干后面追加字符串"""
    return p.parent / (p.stem + s + p.suffix)


def name_with_parents(path: StrPath, num_parents: int) -> Option[str]:
    """路径转换成文件名, 文件名中带有指定数量的上级目录, REMARK: 避免上级目录中包含根目录"""
    parts = list(Path(path).parts)
    start = len(parts) - 1 - num_parents
    if start < 0:
        return Null
    name = "_".join(parts[start:])
    return Some(name)


def replace_home(p: StrPath) -> Path:
    """替换~为HOME目录"""
    p = str(p).replace("~", str(Path.home()))
    return Path(p)


def remove_parent_prefix(p: StrPath) -> Result[Path, str]:
    """从文件名中去掉所在目录名前缀"""
    p = Path(p)
    if p.name.startswith(p.parent.name):
        p1 = p.parent / p.name[len(p.parent.name) + 1 :]
        return Ok(p1)
    return Err("No prefix")


def du(path: StrPath) -> int:
    s = sh.du("-s", path)
    size, _ = parse("{}\t{}", s)
    return int(size)



class Array(list[int]):
    """可以继承自 List"""

    def __init__(self, arr):
        super().__init__()
        self.extend(arr)

    def hi(self):
        print("hi array")


def test_array():
    arr = [1, 2, 4, 8, 16]

    a = Array(arr)
    a.extend(arr)
    print(a)
    a.hi()



from enum import IntEnum


class Key(IntEnum):
    BACKSPACE = 8
    ENTER = 13
    ESC = 27
    BLANK = 32

    D0 = 48
    D1 = 49
    D2 = 50
    D3 = 51
    D4 = 52
    D5 = 53
    D6 = 54
    D7 = 55
    D8 = 56
    D9 = 57

    LEFT = 81
    UP = 82
    RIGHT = 83
    DOWN = 84
    PAGE_UP = 85
    PAGE_DOWN = 86
    INS = 99
    F1 = 190
    F2 = 191
    F3 = 192
    F4 = 193
    F5 = 194
    F6 = 195
    F7 = 196
    F8 = 197
    F9 = 198
    F10 = 199
    DEL = 255


class Flag(IntEnum):
    CTRL = 8
    ALT = 32
    SHIFT = 16


def test_key():
    assert Key.ESC == 27
    assert Key.D9 - Key.D0 == 9



from typing import Any
from urllib.parse import urlparse

import paho.mqtt.publish as publish  # type: ignore
from jcx.net.mqtt.cfg import MqttCfg
from jcx.text.txt_json import to_json


class Publisher:

    def __init__(self, cfg: MqttCfg):
        uri = urlparse(cfg.server_url)
        strs = uri.netloc.split(":", 1)
        self.params = {
            "transport": uri.scheme,
            "hostname": strs[0],
            "port": int(strs[1]),
        }
        self.root_topic = cfg.root_topic

    def publish(self, topic: str, msg: Any) -> None:
        s = to_json(msg)
        publish.single(self.root_topic + "/" + topic, s, **self.params)
